package com.hys.app.service.erp.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.EnumUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.hys.app.converter.erp.EnterpriseConverter;
import com.hys.app.framework.database.WebPage;
import com.hys.app.framework.database.mybatisplus.base.BaseServiceImpl;
import com.hys.app.framework.exception.ServiceException;
import com.hys.app.framework.util.BeanUtils;
import com.hys.app.mapper.erp.EnterpriseMapper;
import com.hys.app.mapper.erp.ProductStockMapper;
import com.hys.app.model.erp.dos.*;
import com.hys.app.model.erp.dto.EnterpriseDTO;
import com.hys.app.model.erp.dto.EnterpriseQueryParams;
import com.hys.app.model.erp.enums.ScanFormCodeTypeEnum;
import com.hys.app.model.erp.vo.EnterpriseVO;
import com.hys.app.model.erp.vo.GoodsVO;
import com.hys.app.model.erp.vo.ProductStockVO;
import com.hys.app.model.erp.vo.ProductVO;
import com.hys.app.service.erp.*;
import com.hys.app.util.ValidateUtil;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 企业业务层实现
 *
 * @author 张崧
 * 2024-03-25 15:44:10
 */
@Service
@AllArgsConstructor
public class ScanCodeManagerImpl extends BaseServiceImpl<ProductStockMapper, ProductStockDO> implements ScanCodeManager {

    private OrderSendManager orderSendManager;
    private OrderItemManager orderItemManager;
    private OrderManager orderManager;
    private WarehouseOutManager warehouseOutManager;
    private WarehouseOutItemManager warehouseOutItemManager;
    private StockDamageReportManager stockDamageReportManager;
    private StockDamageReportProductManager stockDamageReportProductManager;
    private StockInventoryManager stockInventoryManager;
    private StockInventoryProductManager stockInventoryProductManager;
    private OrderReturnManager orderReturnManager;
    private OrderReturnItemManager orderReturnItemManager;
    private StockTransferManager stockTransferManager;
    private StockTransferProductManager stockTransferProductManager;
    private WarehouseEntryManager warehouseEntryManager;
    private WarehouseEntryProductManager warehouseEntryProductManager;
    private ProductStockManager productStockManager;
    private GoodsManager goodsManager;
    private ProductManager productManager;


    @Override
    public Map<String, Object> getListByScan( String orderCode) {
        Map<String, Object> resMap = new HashMap<>();
        List<ProductStockVO> list = new ArrayList<>();

        //自定义的订单号长度都是8为以上，前面8为使我们自定义的用来匹配
        if (orderCode.length() < 8) {
            return list;
        }
        String substring = orderCode.substring(0, 8);
        if (!EnumUtil.contains(ScanFormCodeTypeEnum.class, substring)) {
            return list;
        }
        //发货单
        if (ScanFormCodeTypeEnum.RYKS_FHD.getValue().equals(substring)) {
            OrderSendDO sendDO = orderSendManager.getOne(new LambdaQueryWrapper<OrderSendDO>().eq(OrderSendDO::getSn, orderCode));
            if (ObjectUtil.isNull(sendDO)) {
                return list;
            }
            List<OrderItemDO> orderItemList = orderItemManager.list(new LambdaQueryWrapper<OrderItemDO>().eq(OrderItemDO::getOrderSendId, sendDO.getId()));
            List<Long> collect = orderItemList.stream().map(OrderItemDO::getProductStockId).collect(Collectors.toList());
            list = BeanUtils.toBean(productStockManager.list(new LambdaQueryWrapper<ProductStockDO>().in(ProductStockDO::getId, collect)), ProductStockVO.class);
            resMap.put("order",sendDO);
            resMap.put("orderType",ScanFormCodeTypeEnum.RYKS_FHD.getValue());
        }
        //订单
        if (ScanFormCodeTypeEnum.RYKS_DDD.getValue().equals(substring)) {
            OrderDO orderDo = orderManager.getOne(new LambdaQueryWrapper<OrderDO>().eq(OrderDO::getSn, orderCode));
            if (ObjectUtil.isNull(orderDo)) {
                return list;
            }
            List<OrderItemDO> orderItemList = orderItemManager.list(new LambdaQueryWrapper<OrderItemDO>().eq(OrderItemDO::getOrderId, orderDo.getId()));
            List<Long> collect = orderItemList.stream().map(OrderItemDO::getProductStockId).collect(Collectors.toList());
            list = BeanUtils.toBean(productStockManager.list(new LambdaQueryWrapper<ProductStockDO>().in(ProductStockDO::getId, collect)), ProductStockVO.class);
            resMap.put("order",orderDo);
            resMap.put("orderType",ScanFormCodeTypeEnum.RYKS_DDD.getValue());
        }
        //入库单
        if (ScanFormCodeTypeEnum.RYKS_RKD.getValue().equals(substring)) {
            WarehouseOutDO outDo = warehouseOutManager.getOne(new LambdaQueryWrapper<WarehouseOutDO>().eq(WarehouseOutDO::getSn, orderCode));
            if (ObjectUtil.isNull(outDo)) {
                return list;
            }
            List<WarehouseOutItemDO> orderItemList = warehouseOutItemManager.list(new LambdaQueryWrapper<WarehouseOutItemDO>().eq(WarehouseOutItemDO::getWarehouseOutId, outDo.getId()));
            List<Long> collect = orderItemList.stream().map(WarehouseOutItemDO::getProductStockId).collect(Collectors.toList());
            list = BeanUtils.toBean(productStockManager.list(new LambdaQueryWrapper<ProductStockDO>().in(ProductStockDO::getId, collect)), ProductStockVO.class);
            resMap.put("order",outDo);
            resMap.put("orderType",ScanFormCodeTypeEnum.RYKS_RKD.getValue());
        }
        //盘点
        if (ScanFormCodeTypeEnum.RYKS_PDD.getValue().equals(substring)) {
            StockInventory stockDo = stockInventoryManager.getOne(new LambdaQueryWrapper<StockInventory>().eq(StockInventory::getSn, orderCode));
            if (ObjectUtil.isNull(stockDo)) {
                return list;
            }
            List<StockInventoryProduct> orderItemList = stockInventoryProductManager.list(new LambdaQueryWrapper<StockInventoryProduct>().eq(StockInventoryProduct::getInventoryId, stockDo.getId()));
            List<Long> collect = orderItemList.stream().map(StockInventoryProduct::getProductStockId).collect(Collectors.toList());
            list = BeanUtils.toBean(productStockManager.list(new LambdaQueryWrapper<ProductStockDO>().in(ProductStockDO::getId, collect)), ProductStockVO.class);
            resMap.put("order",stockDo);
            resMap.put("orderType",ScanFormCodeTypeEnum.RYKS_PDD.getValue());
        }
        //退货单
        if (ScanFormCodeTypeEnum.RYKS_THD.getValue().equals(substring)) {
            OrderReturnDO returnDo = orderReturnManager.getOne(new LambdaQueryWrapper<OrderReturnDO>().eq(OrderReturnDO::getSn, orderCode));
            if (ObjectUtil.isNull(returnDo)) {
                return list;
            }
            List<OrderReturnItemDO> orderItemList = orderReturnItemManager.list(new LambdaQueryWrapper<OrderReturnItemDO>().eq(OrderReturnItemDO::getOrderReturnId, returnDo.getId()));
            List<Long> collect = orderItemList.stream().map(OrderReturnItemDO::getProductStockId).collect(Collectors.toList());
            list = BeanUtils.toBean(productStockManager.list(new LambdaQueryWrapper<ProductStockDO>().in(ProductStockDO::getId, collect)), ProductStockVO.class);
            resMap.put("order",returnDo);
            resMap.put("orderType",ScanFormCodeTypeEnum.RYKS_THD.getValue());
        }

        if (ScanFormCodeTypeEnum.RYKS_DBD.getValue().equals(substring)) {
            StockTransferDO transDo = stockTransferManager.getOne(new LambdaQueryWrapper<StockTransferDO>().eq(StockTransferDO::getSn, orderCode));
            if (ObjectUtil.isNull(transDo)) {
                return list;
            }
            List<StockTransferProductDO> orderItemList = stockTransferProductManager.list(new LambdaQueryWrapper<StockTransferProductDO>().eq(StockTransferProductDO::getStockTransferId, transDo.getId()));
            List<Long> collect = orderItemList.stream().map(StockTransferProductDO::getProductStockId).collect(Collectors.toList());
            list = BeanUtils.toBean(productStockManager.list(new LambdaQueryWrapper<ProductStockDO>().in(ProductStockDO::getId, collect)), ProductStockVO.class);
            resMap.put("order",transDo);
            resMap.put("orderType",ScanFormCodeTypeEnum.RYKS_DBD.getValue());
        }

        List<WarehouseEntryDO> list1 = warehouseEntryManager.list();

        //入库单
        if (ScanFormCodeTypeEnum.RYKS_CKD.getValue().equals(substring)) {
            WarehouseEntryDO entryDo = warehouseEntryManager.getOne(new LambdaQueryWrapper<WarehouseEntryDO>().eq(WarehouseEntryDO::getSn, orderCode));
            if (ObjectUtil.isNull(entryDo)) {
                return list;
            }
            List<WarehouseEntryProductDO> orderItemList = warehouseEntryProductManager.list(new LambdaQueryWrapper<WarehouseEntryProductDO>().eq(WarehouseEntryProductDO::getWarehouseEntryId, entryDo.getId()));
            List<Long> collect = orderItemList.stream().map(WarehouseEntryProductDO::getProductStockId).collect(Collectors.toList());
            list = BeanUtils.toBean(productStockManager.list(new LambdaQueryWrapper<ProductStockDO>().in(ProductStockDO::getId, collect)), ProductStockVO.class);
            resMap.put("order",entryDo);
            resMap.put("orderType",ScanFormCodeTypeEnum.RYKS_CKD.getValue());
        }


        if (ScanFormCodeTypeEnum.RYKS_BSD.getValue().equals(substring)) {
            StockDamageReport damageDo = stockDamageReportManager.getOne(new LambdaQueryWrapper<StockDamageReport>().eq(StockDamageReport::getSn, orderCode));
            if (ObjectUtil.isNull(damageDo)) {
                return list;
            }
            List<StockDamageReportProduct> orderItemList = stockDamageReportProductManager.list(new LambdaQueryWrapper<StockDamageReportProduct>().eq(StockDamageReportProduct::getReportId, damageDo.getId()));
            List<Long> collect = orderItemList.stream().map(StockDamageReportProduct::getProductStockId).collect(Collectors.toList());
            list = BeanUtils.toBean(productStockManager.list(new LambdaQueryWrapper<ProductStockDO>().in(ProductStockDO::getId, collect)), ProductStockVO.class);
            resMap.put("order",damageDo);
            resMap.put("orderType",ScanFormCodeTypeEnum.RYKS_BSD.getValue());
        }
        for (ProductStockVO vo : list) {
            vo.setGoodVO(BeanUtil.copyProperties(goodsManager.getById(vo.getGoodsId()), GoodsVO.class));
            vo.setProductVO(BeanUtil.copyProperties(productManager.getById(vo.getProductId()), ProductVO.class));
        }
        resMap.put("list",list);
        return resMap;
    }
}

